// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "UICanvas.hpp"
#include "display/pass/RenderPass.hpp"
#include "display/mesh/QuadMesh.hpp"
#include "ui/UIComponent.hpp"

using namespace Dawn;

void UICanvas::onInit() {
  mesh = std::make_shared<Mesh>();
  mesh->createBuffers(
    QUAD_VERTICE_COUNT * UI_SHADER_QUAD_COUNT,
    QUAD_INDICE_COUNT * UI_SHADER_QUAD_COUNT
  );

  for(int32_t i = 0; i < UI_SHADER_QUAD_COUNT; i++) {
    QuadMesh::bufferWithIndex(
      mesh,
      glm::vec4(0, 0, 1, 1),
      glm::vec4(0, 0, 1, 1),
      i * QUAD_VERTICE_COUNT,
      i * QUAD_INDICE_COUNT,
      i * QUAD_VERTICE_COUNT
    );
  }
}

void UICanvas::onDispose() {
  mesh = nullptr;
}

std::vector<std::shared_ptr<IRenderPass>> UICanvas::getPasses(
struct RenderPassContext &ctx
) {
  if(this->components.empty()) return {};

  glm::mat4 projection;
  glm::mat4 view;

  // data.projection = ctx.camera->getProjection();
  // data.view = ctx.camera->getItem()->getWorldTransform();
  // data.model = this->getItem()->getWorldTransform();

  data.projection = glm::ortho(
    0.0f, 1280.0f,
    720.0f, 0.0f,
    0.0f, 1.0f
  );
  data.view = glm::mat4(1.0f);
  data.model = glm::mat4(1.0f);

  this->passes.clear();
  this->textureBindings.clear();
  this->textures.clear();
  quadCount = 0;
  nextBinding = 0;

  // Define the root alignment

  // Get the quads for each component
  auto itComponents = components.begin();
  auto self = std::ref(*this);
  while(itComponents != components.end()) {
    auto component = *itComponents;
    component->getQuads(self);
    ++itComponents;
  }

  // Flush the remaining quads
  flushPass();
  return passes;
}

void UICanvas::addQuad(
  const glm::vec4 quad,
  const glm::vec4 uvs,
  const struct Color color,
  const enum UIShaderQuadStyle style,
  const std::shared_ptr<Texture> text
) {
  glm::vec4 styleData;
  styleData[0] = (float_t)style;

  if(text == nullptr) {
    styleData[1] = -1;
  } else {
    shadertexturebinding_t texture;
    auto bindingIt = textureBindings.find(text);
    if(bindingIt == textureBindings.end()) {
      if(nextBinding >= UI_SHADER_TEXTURE_COUNT) {
        flushPass();
      }
      textureBindings[text] = nextBinding;
      textures[nextBinding] = text;
      data.textures[nextBinding] = nextBinding;
      texture = nextBinding++;
    } else {
      texture = bindingIt->second;
    }
    styleData[1] = (float_t)texture;
  }

  data.quads[quadCount] = {
    quad,
    uvs,
    color,
    styleData
  };
  quadCount++;
  if(quadCount == UI_SHADER_QUAD_COUNT) flushPass();
}

void UICanvas::flushPass() {
  if(quadCount == 0) return;

  auto pass = createRenderPass<UIShader, UIShaderData>(
    std::ref(*this),
    data,
    textures,
    mesh,
    MeshDrawMode::TRIANGLES,
    0,
    quadCount * QUAD_INDICE_COUNT
  );
  passes.push_back(pass);

  quadCount = 0;
  nextBinding = 0;
  textures.clear();
  textureBindings.clear();
}

void UICanvas::addComponent(std::shared_ptr<UIComponent> component) {
  components.push_back(component);
  component->updateAlignment(
    glm::vec2(0, 0),
    glm::vec2(1280, 720),
    1.0f
  );
}