// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/shader/UIShader.hpp"
#include "component/ui/UICanvas.hpp"

namespace Dawn {
  enum class UIAlignmentType {
    START,
    MIDDLE,
    END,
    STRETCH
  };

  enum class UIAlignmentUnit {
    PIXEL,
    SCALE,
    PERCENT,
    RATIO
  };

  class UIComponent {
    protected:
      glm::vec2 position;
      glm::vec2 size;

      /**
       * Virtual method overridden by the UIComponent to get the quads for the
       * component.
       * 
       * @param alignment The alignment of this component.
       * @param ctx The canvas to add the quads to.
       */
      virtual void getSelfQuads(UICanvas &ctx) = 0;

      /**
       * Updates the alignment of this component based on the parent.
       * 
       * @param parentPosition The position of the parent.
       * @param parentSize The size of the parent.
       */
      void updateAlignment(
        const glm::vec2 parentPosition,
        const glm::vec2 parentSize,
        const float_t canvasScale
      );

    public:
      glm::vec4 align = glm::vec4(0, 0, 32, 32);
      enum UIAlignmentType alignX = UIAlignmentType::START;
      enum UIAlignmentType alignY = UIAlignmentType::START;
      enum UIAlignmentUnit alignUnit[4];

      std::function<
        void(const glm::vec2, const glm::vec2)
      > eventAlignmentUpdated;

      /**
       * Instantiates a new UIComponent.
       */
      UIComponent();

      /**
       * Virtual method overridden by the UIComponent to get the children of
       * this component.
       */
      virtual std::vector<std::shared_ptr<UIComponent>> getChildren();

      /**
       * Method called by the UICanvas to get the quads for this component.
       * 
       * @param ctx The canvas to add the quads to.
       */
      void getQuads(UICanvas &ctx);

      friend class UICanvas;
  };
}